package eu.eudat.logic.proxy.config.configloaders;

import com.fasterxml.jackson.databind.ObjectMapper;
import eu.eudat.logic.proxy.config.ExternalUrls;
import eu.eudat.logic.security.customproviders.ConfigurableProvider.entities.ConfigurableProviders;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import java.io.*;
import java.net.URL;
import java.nio.file.Paths;
import java.util.LinkedList;
import java.util.List;

@Service("configLoader")
@Profile({ "production", "staging" })
public class ProductionConfigLoader implements ConfigLoader {

    private ExternalUrls externalUrls;
    private List<String> rdaProperties;
    private XWPFDocument document;
    private ConfigurableProviders configurableProviders;

    @Autowired
    private Environment environment;

    private void setExternalUrls() {
        String fileUrl = this.environment.getProperty("configuration.externalUrls");
        System.out.println("Loaded also config file: " + fileUrl);
        String current = null;
        InputStream is = null;
        try {
            current = new java.io.File(".").getCanonicalPath();

            JAXBContext jaxbContext = JAXBContext.newInstance(ExternalUrls.class);
            Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
            is = new URL(Paths.get(fileUrl).toUri().toURL().toString()).openStream();
            externalUrls = (ExternalUrls) jaxbUnmarshaller.unmarshal(is);

        } catch (Exception ex) {
            ex.printStackTrace();
            System.out.println("Cannot find in folder" + current);
        } finally {
            try {
                if (is != null) is.close();
            } catch (IOException e) {
                System.out.println("Warning: Could not close a stream after reading from file: " + fileUrl);
            }
        }
    }

    private void setRdaProperties() {
        String filePath = environment.getProperty("configuration.rda");
        BufferedReader reader;
        List<String> rdaList = new LinkedList<>();
        try {
            reader = new BufferedReader(new FileReader(filePath));
            String line = reader.readLine();
            while (line != null) {
                rdaList.add(line);
                line = reader.readLine();
            }
            reader.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

        rdaProperties = rdaList;
    }

    private void setDocument() {
        String filePath = environment.getProperty("configuration.h2020template");
        String current = null;
        try {
            current = new java.io.File(".").getCanonicalPath();
            InputStream is = new URL(Paths.get(filePath).toUri().toURL().toString()).openStream();
            this.document = new XWPFDocument(is);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void setConfigurableProviders() {
        String filePath = environment.getProperty("configuration.configurable_login_providers");
        InputStream is = null;
        try {
            File tempFile = new File(filePath);
            if (tempFile.exists()) {
                is = new URL(Paths.get(filePath).toUri().toURL().toString()).openStream();
                ObjectMapper objectMapper = new ObjectMapper();
                this.configurableProviders = objectMapper.readValue(is, ConfigurableProviders.class);
            } else {
                this.configurableProviders = new ConfigurableProviders();
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (is != null) is.close();
            } catch (IOException e) {
                System.out.println("Warning: Could not close a stream after reading from file: " + filePath);
            }
        }
    }

    public ExternalUrls getExternalUrls() {
        this.setExternalUrls();
        return externalUrls;
    }

    public List<String> getRdaProperties() {
        this.setRdaProperties();
        return rdaProperties;
    }

    public XWPFDocument getDocument() {
        this.setDocument();
        return document;
    }

    public ConfigurableProviders getConfigurableProviders() {
        this.setConfigurableProviders();
        return configurableProviders;
    }
}